/****************************************************************************
** Meta object code from reading C++ file 'DockWidgetInstantiator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/qtquick/DockWidgetInstantiator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DockWidgetInstantiator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::DockWidgetInstantiator::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::DockWidgetInstantiator",
        "QML.Element",
        "DockWidget",
        "uniqueNameChanged",
        "",
        "sourceChanged",
        "dockWidgetChanged",
        "actualTitleBarChanged",
        "titleChanged",
        "title",
        "iconChanged",
        "closed",
        "guestViewChanged",
        "QQuickItem*",
        "optionsChanged",
        "KDDockWidgets::DockWidgetOptions",
        "isFocusedChanged",
        "isOverlayedChanged",
        "isFloatingChanged",
        "isOpenChanged",
        "removedFromSideBar",
        "windowActiveAboutToChange",
        "activated",
        "affinitiesChanged",
        "lastCloseReasonChanged",
        "userDataChanged",
        "dockWidget",
        "KDDockWidgets::QtQuick::DockWidget*",
        "controller",
        "KDDockWidgets::Core::DockWidget*",
        "addDockWidgetAsTab",
        "other",
        "KDDockWidgets::InitialVisibilityOption",
        "addDockWidgetToContainingWindow",
        "KDDockWidgets::Location",
        "location",
        "relativeTo",
        "QSize",
        "initialSize",
        "setAsCurrentTab",
        "forceClose",
        "close",
        "open",
        "show",
        "raise",
        "moveToSideBar",
        "deleteDockWidget",
        "deleteDockWidgetLater",
        "uniqueName",
        "source",
        "actualTitleBar",
        "isFocused",
        "isFloating",
        "isOpen",
        "affinities",
        "options",
        "lastCloseReason",
        "KDDockWidgets::CloseReason",
        "userData",
        "QVariantMap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'uniqueNameChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dockWidgetChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'actualTitleBarChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closed'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'guestViewChanged'
        QtMocHelpers::SignalData<void(QQuickItem *)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 4 },
        }}),
        // Signal 'optionsChanged'
        QtMocHelpers::SignalData<void(KDDockWidgets::DockWidgetOptions)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 4 },
        }}),
        // Signal 'isFocusedChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'isOverlayedChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'isFloatingChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'isOpenChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'removedFromSideBar'
        QtMocHelpers::SignalData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowActiveAboutToChange'
        QtMocHelpers::SignalData<void(bool)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Signal 'affinitiesChanged'
        QtMocHelpers::SignalData<void()>(23, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastCloseReasonChanged'
        QtMocHelpers::SignalData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userDataChanged'
        QtMocHelpers::SignalData<void()>(25, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'dockWidget'
        QtMocHelpers::MethodData<KDDockWidgets::QtQuick::DockWidget *() const>(26, 4, QMC::AccessPublic, 0x80000000 | 27),
        // Method 'controller'
        QtMocHelpers::MethodData<KDDockWidgets::Core::DockWidget *() const>(28, 4, QMC::AccessPublic, 0x80000000 | 29),
        // Method 'addDockWidgetAsTab'
        QtMocHelpers::MethodData<void(QQuickItem *, KDDockWidgets::InitialVisibilityOption)>(30, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 31 }, { 0x80000000 | 32, 4 },
        }}),
        // Method 'addDockWidgetAsTab'
        QtMocHelpers::MethodData<void(QQuickItem *)>(30, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 31 },
        }}),
        // Method 'addDockWidgetToContainingWindow'
        QtMocHelpers::MethodData<void(QQuickItem *, KDDockWidgets::Location, QQuickItem *, QSize, KDDockWidgets::InitialVisibilityOption)>(33, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 31 }, { 0x80000000 | 34, 35 }, { 0x80000000 | 13, 36 }, { 0x80000000 | 37, 38 },
            { 0x80000000 | 32, 4 },
        }}),
        // Method 'addDockWidgetToContainingWindow'
        QtMocHelpers::MethodData<void(QQuickItem *, KDDockWidgets::Location, QQuickItem *, QSize)>(33, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 31 }, { 0x80000000 | 34, 35 }, { 0x80000000 | 13, 36 }, { 0x80000000 | 37, 38 },
        }}),
        // Method 'addDockWidgetToContainingWindow'
        QtMocHelpers::MethodData<void(QQuickItem *, KDDockWidgets::Location, QQuickItem *)>(33, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 31 }, { 0x80000000 | 34, 35 }, { 0x80000000 | 13, 36 },
        }}),
        // Method 'addDockWidgetToContainingWindow'
        QtMocHelpers::MethodData<void(QQuickItem *, KDDockWidgets::Location)>(33, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 31 }, { 0x80000000 | 34, 35 },
        }}),
        // Method 'setAsCurrentTab'
        QtMocHelpers::MethodData<void()>(39, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'forceClose'
        QtMocHelpers::MethodData<void()>(40, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'close'
        QtMocHelpers::MethodData<bool()>(41, 4, QMC::AccessPublic, QMetaType::Bool),
        // Method 'open'
        QtMocHelpers::MethodData<void()>(42, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'show'
        QtMocHelpers::MethodData<void()>(43, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'raise'
        QtMocHelpers::MethodData<void()>(44, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'moveToSideBar'
        QtMocHelpers::MethodData<void()>(45, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'deleteDockWidget'
        QtMocHelpers::MethodData<void()>(46, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'deleteDockWidgetLater'
        QtMocHelpers::MethodData<void()>(47, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'uniqueName'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'source'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'dockWidget'
        QtMocHelpers::PropertyData<KDDockWidgets::QtQuick::DockWidget*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'actualTitleBar'
        QtMocHelpers::PropertyData<QObject*>(50, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 3),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isFocused'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'isFloating'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 11),
        // property 'isOpen'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags, 12),
        // property 'affinities'
        QtMocHelpers::PropertyData<QList<QString>>(54, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'options'
        QtMocHelpers::PropertyData<KDDockWidgets::DockWidgetOptions>(55, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'lastCloseReason'
        QtMocHelpers::PropertyData<KDDockWidgets::CloseReason>(56, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 16),
        // property 'userData'
        QtMocHelpers::PropertyData<QVariantMap>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<DockWidgetInstantiator, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13KDDockWidgets22DockWidgetInstantiatorE[] = {
    QMetaObject::SuperData::link<KDDockWidgets::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KDDockWidgets::DockWidgetInstantiator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13KDDockWidgets22DockWidgetInstantiatorE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::DockWidgetInstantiator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockWidgetInstantiator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->uniqueNameChanged(); break;
        case 1: _t->sourceChanged(); break;
        case 2: _t->dockWidgetChanged(); break;
        case 3: _t->actualTitleBarChanged(); break;
        case 4: _t->titleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->iconChanged(); break;
        case 6: _t->closed(); break;
        case 7: _t->guestViewChanged((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 8: _t->optionsChanged((*reinterpret_cast<std::add_pointer_t<KDDockWidgets::DockWidgetOptions>>(_a[1]))); break;
        case 9: _t->isFocusedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->isOverlayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->isFloatingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->isOpenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->removedFromSideBar(); break;
        case 14: _t->windowActiveAboutToChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->affinitiesChanged(); break;
        case 16: _t->lastCloseReasonChanged(); break;
        case 17: _t->userDataChanged(); break;
        case 18: { KDDockWidgets::QtQuick::DockWidget* _r = _t->dockWidget();
            if (_a[0]) *reinterpret_cast<KDDockWidgets::QtQuick::DockWidget**>(_a[0]) = std::move(_r); }  break;
        case 19: { KDDockWidgets::Core::DockWidget* _r = _t->controller();
            if (_a[0]) *reinterpret_cast<KDDockWidgets::Core::DockWidget**>(_a[0]) = std::move(_r); }  break;
        case 20: _t->addDockWidgetAsTab((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::InitialVisibilityOption>>(_a[2]))); break;
        case 21: _t->addDockWidgetAsTab((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 22: _t->addDockWidgetToContainingWindow((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::Location>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::InitialVisibilityOption>>(_a[5]))); break;
        case 23: _t->addDockWidgetToContainingWindow((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::Location>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[4]))); break;
        case 24: _t->addDockWidgetToContainingWindow((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::Location>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[3]))); break;
        case 25: _t->addDockWidgetToContainingWindow((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDDockWidgets::Location>>(_a[2]))); break;
        case 26: _t->setAsCurrentTab(); break;
        case 27: _t->forceClose(); break;
        case 28: { bool _r = _t->close();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->open(); break;
        case 30: _t->show(); break;
        case 31: _t->raise(); break;
        case 32: _t->moveToSideBar(); break;
        case 33: _t->deleteDockWidget(); break;
        case 34: _t->deleteDockWidgetLater(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::InitialVisibilityOption >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::InitialVisibilityOption >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::Location >(); break;
            case 2:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::Location >(); break;
            case 2:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::Location >(); break;
            case 2:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDDockWidgets::Location >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::uniqueNameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::sourceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::dockWidgetChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::actualTitleBarChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(const QString & )>(_a, &DockWidgetInstantiator::titleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::iconChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::closed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(QQuickItem * )>(_a, &DockWidgetInstantiator::guestViewChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(KDDockWidgets::DockWidgetOptions )>(_a, &DockWidgetInstantiator::optionsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(bool )>(_a, &DockWidgetInstantiator::isFocusedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(bool )>(_a, &DockWidgetInstantiator::isOverlayedChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(bool )>(_a, &DockWidgetInstantiator::isFloatingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(bool )>(_a, &DockWidgetInstantiator::isOpenChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::removedFromSideBar, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)(bool )>(_a, &DockWidgetInstantiator::windowActiveAboutToChange, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::affinitiesChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::lastCloseReasonChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidgetInstantiator::*)()>(_a, &DockWidgetInstantiator::userDataChanged, 17))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDDockWidgets::QtQuick::DockWidget* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->uniqueName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->source(); break;
        case 2: *reinterpret_cast<KDDockWidgets::QtQuick::DockWidget**>(_v) = _t->dockWidget(); break;
        case 3: *reinterpret_cast<QObject**>(_v) = _t->actualTitleBar(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isFocused(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isFloating(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isOpen(); break;
        case 8: *reinterpret_cast<QList<QString>*>(_v) = _t->affinities(); break;
        case 9: *reinterpret_cast<KDDockWidgets::DockWidgetOptions*>(_v) = _t->options(); break;
        case 10: *reinterpret_cast<KDDockWidgets::CloseReason*>(_v) = _t->lastCloseReason(); break;
        case 11: *reinterpret_cast<QVariantMap*>(_v) = _t->userData(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUniqueName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setSource(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setFloating(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setAffinities(*reinterpret_cast<QList<QString>*>(_v)); break;
        case 9: _t->setOptions(*reinterpret_cast<KDDockWidgets::DockWidgetOptions*>(_v)); break;
        case 11: _t->setUserData(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KDDockWidgets::DockWidgetInstantiator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::DockWidgetInstantiator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets22DockWidgetInstantiatorE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int KDDockWidgets::DockWidgetInstantiator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void KDDockWidgets::DockWidgetInstantiator::uniqueNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KDDockWidgets::DockWidgetInstantiator::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KDDockWidgets::DockWidgetInstantiator::dockWidgetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KDDockWidgets::DockWidgetInstantiator::actualTitleBarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KDDockWidgets::DockWidgetInstantiator::titleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KDDockWidgets::DockWidgetInstantiator::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KDDockWidgets::DockWidgetInstantiator::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KDDockWidgets::DockWidgetInstantiator::guestViewChanged(QQuickItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KDDockWidgets::DockWidgetInstantiator::optionsChanged(KDDockWidgets::DockWidgetOptions _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KDDockWidgets::DockWidgetInstantiator::isFocusedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void KDDockWidgets::DockWidgetInstantiator::isOverlayedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KDDockWidgets::DockWidgetInstantiator::isFloatingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void KDDockWidgets::DockWidgetInstantiator::isOpenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void KDDockWidgets::DockWidgetInstantiator::removedFromSideBar()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KDDockWidgets::DockWidgetInstantiator::windowActiveAboutToChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KDDockWidgets::DockWidgetInstantiator::affinitiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void KDDockWidgets::DockWidgetInstantiator::lastCloseReasonChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void KDDockWidgets::DockWidgetInstantiator::userDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
