/****************************************************************************
** Meta object code from reading C++ file 'DockWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/qtquick/views/DockWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DockWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::QtQuick::DockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::QtQuick::DockWidget",
        "groupGeometryChanged",
        "",
        "QRect",
        "actualTitleBarChanged",
        "isFocusedChanged",
        "isOpenChanged",
        "isFloatingChanged",
        "titleChanged",
        "guestItemChanged",
        "optionsChanged",
        "setGuestItem",
        "QQuickItem*",
        "onGeometryUpdated",
        "toggleAction",
        "KDDockWidgets::QtQuick::Action*",
        "floatAction",
        "setAsCurrentTab",
        "forceClose",
        "open",
        "show",
        "raise",
        "moveToSideBar",
        "actualTitleBar",
        "isFocused",
        "isFloating",
        "isOpen",
        "uniqueName",
        "title",
        "guestItem",
        "options",
        "KDDockWidgets::DockWidgetOptions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'groupGeometryChanged'
        QtMocHelpers::SignalData<void(QRect)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'actualTitleBarChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isFocusedChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isOpenChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'isFloatingChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'guestItemChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'optionsChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setGuestItem'
        QtMocHelpers::MethodData<void(QQuickItem *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Method 'onGeometryUpdated'
        QtMocHelpers::MethodData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'toggleAction'
        QtMocHelpers::MethodData<KDDockWidgets::QtQuick::Action *() const>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Method 'floatAction'
        QtMocHelpers::MethodData<KDDockWidgets::QtQuick::Action *() const>(16, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Method 'setAsCurrentTab'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'forceClose'
        QtMocHelpers::MethodData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'open'
        QtMocHelpers::MethodData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'show'
        QtMocHelpers::MethodData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'raise'
        QtMocHelpers::MethodData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'moveToSideBar'
        QtMocHelpers::MethodData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'actualTitleBar'
        QtMocHelpers::PropertyData<QObject*>(23, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 1),
        // property 'isFocused'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'isFloating'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'isOpen'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'uniqueName'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'guestItem'
        QtMocHelpers::PropertyData<QObject*>(29, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 6),
        // property 'options'
        QtMocHelpers::PropertyData<KDDockWidgets::DockWidgetOptions>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DockWidget, qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13KDDockWidgets7QtQuick10DockWidgetE[] = {
    QMetaObject::SuperData::link<KDDockWidgets::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KDDockWidgets::QtQuick::DockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QtQuick::View::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13KDDockWidgets7QtQuick10DockWidgetE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::QtQuick::DockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->groupGeometryChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 1: _t->actualTitleBarChanged(); break;
        case 2: _t->isFocusedChanged(); break;
        case 3: _t->isOpenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->isFloatingChanged(); break;
        case 5: _t->titleChanged(); break;
        case 6: _t->guestItemChanged(); break;
        case 7: _t->optionsChanged(); break;
        case 8: _t->setGuestItem((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 9: _t->onGeometryUpdated(); break;
        case 10: { KDDockWidgets::QtQuick::Action* _r = _t->toggleAction();
            if (_a[0]) *reinterpret_cast<KDDockWidgets::QtQuick::Action**>(_a[0]) = std::move(_r); }  break;
        case 11: { KDDockWidgets::QtQuick::Action* _r = _t->floatAction();
            if (_a[0]) *reinterpret_cast<KDDockWidgets::QtQuick::Action**>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setAsCurrentTab(); break;
        case 13: _t->forceClose(); break;
        case 14: _t->open(); break;
        case 15: _t->show(); break;
        case 16: _t->raise(); break;
        case 17: _t->moveToSideBar(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)(QRect )>(_a, &DockWidget::groupGeometryChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::actualTitleBarChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::isFocusedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)(bool )>(_a, &DockWidget::isOpenChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::isFloatingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::titleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::guestItemChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockWidget::*)()>(_a, &DockWidget::optionsChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QObject**>(_v) = _t->actualTitleBarView(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isFocused(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isFloating(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isOpen(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->uniqueName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 6: *reinterpret_cast<QObject**>(_v) = _t->guestItem(); break;
        case 7: *reinterpret_cast<KDDockWidgets::DockWidgetOptions*>(_v) = _t->options(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setFloating(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setOptions(*reinterpret_cast<KDDockWidgets::DockWidgetOptions*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KDDockWidgets::QtQuick::DockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::QtQuick::DockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick10DockWidgetE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DockWidgetViewInterface"))
        return static_cast< Core::DockWidgetViewInterface*>(this);
    return QtQuick::View::qt_metacast(_clname);
}

int KDDockWidgets::QtQuick::DockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtQuick::View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KDDockWidgets::QtQuick::DockWidget::groupGeometryChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KDDockWidgets::QtQuick::DockWidget::actualTitleBarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KDDockWidgets::QtQuick::DockWidget::isFocusedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KDDockWidgets::QtQuick::DockWidget::isOpenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KDDockWidgets::QtQuick::DockWidget::isFloatingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KDDockWidgets::QtQuick::DockWidget::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KDDockWidgets::QtQuick::DockWidget::guestItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KDDockWidgets::QtQuick::DockWidget::optionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
