/****************************************************************************
** Meta object code from reading C++ file 'KDDockWidgets.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/KDDockWidgets.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KDDockWidgets.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgetsE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgetsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets",
        "QML.Element",
        "auto",
        "Location",
        "Location_None",
        "Location_OnLeft",
        "Location_OnTop",
        "Location_OnRight",
        "Location_OnBottom",
        "MainWindowOptions",
        "MainWindowOption",
        "MainWindowOption_None",
        "MainWindowOption_HasCentralGroup",
        "MainWindowOption_HasCentralFrame",
        "MainWindowOption_MDI",
        "MainWindowOption_HasCentralWidget",
        "MainWindowOption_QDockWidgets",
        "MainWindowOption_ManualInit",
        "MainWindowOption_CentralWidgetGetsAllExtraSpace",
        "DockWidgetOptions",
        "DockWidgetOption",
        "DockWidgetOption_None",
        "DockWidgetOption_NotClosable",
        "DockWidgetOption_NotDockable",
        "DockWidgetOption_DeleteOnClose",
        "DockWidgetOption_MDINestable",
        "DockWidgetOption_NoDrops",
        "IconPlace",
        "TitleBar",
        "TabBar",
        "ToggleAction",
        "All",
        "FrontendType",
        "QtWidgets",
        "QtQuick",
        "Flutter",
        "DefaultSizeMode",
        "ItemSize",
        "Fair",
        "FairButFloor",
        "NoDefaultSizeMode",
        "AddingOption",
        "AddingOption_None",
        "AddingOption_StartHidden",
        "InitialVisibilityOption",
        "StartVisible",
        "StartHidden",
        "PreserveCurrentTab",
        "NeighbourSqueezeStrategy",
        "AllNeighbours",
        "ImmediateNeighboursFirst",
        "RestoreOptions",
        "RestoreOption",
        "RestoreOption_None",
        "RestoreOption_RelativeToMainWindow",
        "RestoreOption_AbsoluteFloatingDockWindows",
        "DropIndicatorType",
        "Classic",
        "Segmented",
        "None",
        "SuggestedGeometryHint",
        "SuggestedGeometryHint_None",
        "SuggestedGeometryHint_PreserveCenter",
        "SuggestedGeometryHint_GeometryIsFromDocked",
        "TitleBarButtonType",
        "Close",
        "Float",
        "Minimize",
        "Maximize",
        "Normal",
        "AutoHide",
        "UnautoHide",
        "AllTitleBarButtonTypes",
        "DropLocation",
        "DropLocation_None",
        "DropLocation_Left",
        "DropLocation_Top",
        "DropLocation_Right",
        "DropLocation_Bottom",
        "DropLocation_Center",
        "DropLocation_OutterLeft",
        "DropLocation_OutterTop",
        "DropLocation_OutterRight",
        "DropLocation_OutterBottom",
        "DropLocation_Inner",
        "DropLocation_Outter",
        "DropLocation_Horizontal",
        "DropLocation_Vertical",
        "CursorPosition",
        "CursorPosition_Undefined",
        "CursorPosition_Left",
        "CursorPosition_Right",
        "CursorPosition_Top",
        "CursorPosition_Bottom",
        "CursorPosition_TopLeft",
        "CursorPosition_TopRight",
        "CursorPosition_BottomRight",
        "CursorPosition_BottomLeft",
        "CursorPosition_Horizontal",
        "CursorPosition_Vertical",
        "CursorPosition_All",
        "FrameOptions",
        "FrameOption",
        "FrameOption_None",
        "FrameOption_AlwaysShowsTabs",
        "FrameOption_IsCentralFrame",
        "FrameOption_IsOverlayed",
        "FrameOption_NonDockable",
        "StackOptions",
        "StackOption",
        "StackOption_None",
        "StackOption_DocumentMode",
        "CloseReason",
        "Unspecified",
        "TitleBarCloseButton",
        "Action",
        "MovedToSideBar",
        "OverlayCollapse"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Location'
        QtMocHelpers::EnumData<Location>(3, 3, QMC::EnumFlags{}).add({
            {    4, Location::Location_None },
            {    5, Location::Location_OnLeft },
            {    6, Location::Location_OnTop },
            {    7, Location::Location_OnRight },
            {    8, Location::Location_OnBottom },
        }),
        // enum 'MainWindowOptions'
        QtMocHelpers::EnumData<MainWindowOptions>(9, 10, QMC::EnumFlags{}).add({
            {   11, MainWindowOption::MainWindowOption_None },
            {   12, MainWindowOption::MainWindowOption_HasCentralGroup },
            {   13, MainWindowOption::MainWindowOption_HasCentralFrame },
            {   14, MainWindowOption::MainWindowOption_MDI },
            {   15, MainWindowOption::MainWindowOption_HasCentralWidget },
            {   16, MainWindowOption::MainWindowOption_QDockWidgets },
            {   17, MainWindowOption::MainWindowOption_ManualInit },
            {   18, MainWindowOption::MainWindowOption_CentralWidgetGetsAllExtraSpace },
        }),
        // enum 'DockWidgetOptions'
        QtMocHelpers::EnumData<DockWidgetOptions>(19, 20, QMC::EnumFlags{}).add({
            {   21, DockWidgetOption::DockWidgetOption_None },
            {   22, DockWidgetOption::DockWidgetOption_NotClosable },
            {   23, DockWidgetOption::DockWidgetOption_NotDockable },
            {   24, DockWidgetOption::DockWidgetOption_DeleteOnClose },
            {   25, DockWidgetOption::DockWidgetOption_MDINestable },
            {   26, DockWidgetOption::DockWidgetOption_NoDrops },
        }),
        // enum 'IconPlace'
        QtMocHelpers::EnumData<IconPlace>(27, 27, QMC::EnumIsScoped).add({
            {   28, IconPlace::TitleBar },
            {   29, IconPlace::TabBar },
            {   30, IconPlace::ToggleAction },
            {   31, IconPlace::All },
        }),
        // enum 'FrontendType'
        QtMocHelpers::EnumData<FrontendType>(32, 32, QMC::EnumIsScoped).add({
            {   33, FrontendType::QtWidgets },
            {   34, FrontendType::QtQuick },
            {   35, FrontendType::Flutter },
        }),
        // enum 'DefaultSizeMode'
        QtMocHelpers::EnumData<DefaultSizeMode>(36, 36, QMC::EnumIsScoped).add({
            {   37, DefaultSizeMode::ItemSize },
            {   38, DefaultSizeMode::Fair },
            {   39, DefaultSizeMode::FairButFloor },
            {   40, DefaultSizeMode::NoDefaultSizeMode },
        }),
        // enum 'AddingOption'
        QtMocHelpers::EnumData<AddingOption>(41, 41, QMC::EnumFlags{}).add({
            {   42, AddingOption::AddingOption_None },
            {   43, AddingOption::AddingOption_StartHidden },
        }),
        // enum 'InitialVisibilityOption'
        QtMocHelpers::EnumData<InitialVisibilityOption>(44, 44, QMC::EnumIsScoped).add({
            {   45, InitialVisibilityOption::StartVisible },
            {   46, InitialVisibilityOption::StartHidden },
            {   47, InitialVisibilityOption::PreserveCurrentTab },
        }),
        // enum 'NeighbourSqueezeStrategy'
        QtMocHelpers::EnumData<NeighbourSqueezeStrategy>(48, 48, QMC::EnumIsScoped).add({
            {   49, NeighbourSqueezeStrategy::AllNeighbours },
            {   50, NeighbourSqueezeStrategy::ImmediateNeighboursFirst },
        }),
        // enum 'RestoreOptions'
        QtMocHelpers::EnumData<RestoreOptions>(51, 52, QMC::EnumFlags{}).add({
            {   53, RestoreOption::RestoreOption_None },
            {   54, RestoreOption::RestoreOption_RelativeToMainWindow },
            {   55, RestoreOption::RestoreOption_AbsoluteFloatingDockWindows },
        }),
        // enum 'DropIndicatorType'
        QtMocHelpers::EnumData<DropIndicatorType>(56, 56, QMC::EnumIsScoped).add({
            {   57, DropIndicatorType::Classic },
            {   58, DropIndicatorType::Segmented },
            {   59, DropIndicatorType::None },
        }),
        // enum 'SuggestedGeometryHint'
        QtMocHelpers::EnumData<SuggestedGeometryHint>(60, 60, QMC::EnumFlags{}).add({
            {   61, SuggestedGeometryHint::SuggestedGeometryHint_None },
            {   62, SuggestedGeometryHint::SuggestedGeometryHint_PreserveCenter },
            {   63, SuggestedGeometryHint::SuggestedGeometryHint_GeometryIsFromDocked },
        }),
        // enum 'TitleBarButtonType'
        QtMocHelpers::EnumData<TitleBarButtonType>(64, 64, QMC::EnumIsScoped).add({
            {   65, TitleBarButtonType::Close },
            {   66, TitleBarButtonType::Float },
            {   67, TitleBarButtonType::Minimize },
            {   68, TitleBarButtonType::Maximize },
            {   69, TitleBarButtonType::Normal },
            {   70, TitleBarButtonType::AutoHide },
            {   71, TitleBarButtonType::UnautoHide },
            {   72, TitleBarButtonType::AllTitleBarButtonTypes },
        }),
        // enum 'DropLocation'
        QtMocHelpers::EnumData<DropLocation>(73, 73, QMC::EnumFlags{}).add({
            {   74, DropLocation::DropLocation_None },
            {   75, DropLocation::DropLocation_Left },
            {   76, DropLocation::DropLocation_Top },
            {   77, DropLocation::DropLocation_Right },
            {   78, DropLocation::DropLocation_Bottom },
            {   79, DropLocation::DropLocation_Center },
            {   80, DropLocation::DropLocation_OutterLeft },
            {   81, DropLocation::DropLocation_OutterTop },
            {   82, DropLocation::DropLocation_OutterRight },
            {   83, DropLocation::DropLocation_OutterBottom },
            {   84, DropLocation::DropLocation_Inner },
            {   85, DropLocation::DropLocation_Outter },
            {   86, DropLocation::DropLocation_Horizontal },
            {   87, DropLocation::DropLocation_Vertical },
        }),
        // enum 'CursorPosition'
        QtMocHelpers::EnumData<CursorPosition>(88, 88, QMC::EnumFlags{}).add({
            {   89, CursorPosition::CursorPosition_Undefined },
            {   90, CursorPosition::CursorPosition_Left },
            {   91, CursorPosition::CursorPosition_Right },
            {   92, CursorPosition::CursorPosition_Top },
            {   93, CursorPosition::CursorPosition_Bottom },
            {   94, CursorPosition::CursorPosition_TopLeft },
            {   95, CursorPosition::CursorPosition_TopRight },
            {   96, CursorPosition::CursorPosition_BottomRight },
            {   97, CursorPosition::CursorPosition_BottomLeft },
            {   98, CursorPosition::CursorPosition_Horizontal },
            {   99, CursorPosition::CursorPosition_Vertical },
            {  100, CursorPosition::CursorPosition_All },
        }),
        // enum 'FrameOptions'
        QtMocHelpers::EnumData<FrameOptions>(101, 102, QMC::EnumFlags{}).add({
            {  103, FrameOption::FrameOption_None },
            {  104, FrameOption::FrameOption_AlwaysShowsTabs },
            {  105, FrameOption::FrameOption_IsCentralFrame },
            {  106, FrameOption::FrameOption_IsOverlayed },
            {  107, FrameOption::FrameOption_NonDockable },
        }),
        // enum 'StackOptions'
        QtMocHelpers::EnumData<StackOptions>(108, 109, QMC::EnumFlags{}).add({
            {  110, StackOption::StackOption_None },
            {  111, StackOption::StackOption_DocumentMode },
        }),
        // enum 'CloseReason'
        QtMocHelpers::EnumData<CloseReason>(112, 112, QMC::EnumIsScoped).add({
            {  113, CloseReason::Unspecified },
            {  114, CloseReason::TitleBarCloseButton },
            {  115, CloseReason::Action },
            {  116, CloseReason::MovedToSideBar },
            {  117, CloseReason::OverlayCollapse },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN13KDDockWidgetsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN13KDDockWidgetsE =
    KDDockWidgets::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgetsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN13KDDockWidgetsE =
    qt_staticMetaObjectContent_ZN13KDDockWidgetsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN13KDDockWidgetsE =
    qt_staticMetaObjectContent_ZN13KDDockWidgetsE.relocatingData;

Q_CONSTINIT const QMetaObject KDDockWidgets::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN13KDDockWidgetsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN13KDDockWidgetsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN13KDDockWidgetsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
