/****************************************************************************
** Meta object code from reading C++ file 'TitleBar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/qtquick/views/TitleBar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TitleBar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::QtQuick::TitleBar::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::QtQuick::TitleBar",
        "QML.Element",
        "TitleBarView",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Created by the framework only.",
        "titleBarQmlItemChanged",
        "",
        "titleChanged",
        "iconChanged",
        "isFocusedChanged",
        "closeButtonEnabledChanged",
        "floatButtonVisibleChanged",
        "maximizeButtonVisibleChanged",
        "minimizeButtonVisibleChanged",
        "floatButtonToolTipChanged",
        "numDockWidgetsChanged",
        "onDoubleClicked",
        "onCloseClicked",
        "onFloatClicked",
        "onMinimizeClicked",
        "onAutoHideClicked",
        "isFloating",
        "isMaximized",
        "toggleMaximized",
        "onMaximizeClicked",
        "titleBarQmlItem",
        "QQuickItem*",
        "titleBarMouseArea",
        "title",
        "hasIcon",
        "closeButtonEnabled",
        "floatButtonVisible",
        "maximizeButtonVisible",
        "minimizeButtonVisible",
        "floatButtonToolTip",
        "isFocused",
        "maximizeUsesRestoreIcon"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'titleBarQmlItemChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isFocusedChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeButtonEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'floatButtonVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'maximizeButtonVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'minimizeButtonVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'floatButtonToolTipChanged'
        QtMocHelpers::SignalData<void(const QString &)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'numDockWidgetsChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'onDoubleClicked'
        QtMocHelpers::MethodData<bool()>(18, 8, QMC::AccessProtected, QMetaType::Bool),
        // Method 'onCloseClicked'
        QtMocHelpers::MethodData<void()>(19, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'onFloatClicked'
        QtMocHelpers::MethodData<void()>(20, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'onMinimizeClicked'
        QtMocHelpers::MethodData<void()>(21, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'onAutoHideClicked'
        QtMocHelpers::MethodData<void()>(22, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'isFloating'
        QtMocHelpers::MethodData<bool() const>(23, 8, QMC::AccessProtected, QMetaType::Bool),
        // Method 'isMaximized'
        QtMocHelpers::MethodData<bool() const>(24, 8, QMC::AccessProtected, QMetaType::Bool),
        // Method 'toggleMaximized'
        QtMocHelpers::MethodData<void()>(25, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'onMaximizeClicked'
        QtMocHelpers::MethodData<void()>(26, 8, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'titleBarQmlItem'
        QtMocHelpers::PropertyData<QQuickItem*>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'titleBarMouseArea'
        QtMocHelpers::PropertyData<QQuickItem*>(29, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'hasIcon'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'closeButtonEnabled'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'floatButtonVisible'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'maximizeButtonVisible'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'minimizeButtonVisible'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'floatButtonToolTip'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'isFocused'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'maximizeUsesRestoreIcon'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<TitleBar, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::QtQuick::TitleBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QtQuick::View::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::QtQuick::TitleBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TitleBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->titleBarQmlItemChanged(); break;
        case 1: _t->titleChanged(); break;
        case 2: _t->iconChanged(); break;
        case 3: _t->isFocusedChanged(); break;
        case 4: _t->closeButtonEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->floatButtonVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->maximizeButtonVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->minimizeButtonVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->floatButtonToolTipChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->numDockWidgetsChanged(); break;
        case 10: { bool _r = _t->onDoubleClicked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->onCloseClicked(); break;
        case 12: _t->onFloatClicked(); break;
        case 13: _t->onMinimizeClicked(); break;
        case 14: _t->onAutoHideClicked(); break;
        case 15: { bool _r = _t->isFloating();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isMaximized();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->toggleMaximized(); break;
        case 18: _t->onMaximizeClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)()>(_a, &TitleBar::titleBarQmlItemChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)()>(_a, &TitleBar::titleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)()>(_a, &TitleBar::iconChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)()>(_a, &TitleBar::isFocusedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)(bool )>(_a, &TitleBar::closeButtonEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)(bool )>(_a, &TitleBar::floatButtonVisibleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)(bool )>(_a, &TitleBar::maximizeButtonVisibleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)(bool )>(_a, &TitleBar::minimizeButtonVisibleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)(const QString & )>(_a, &TitleBar::floatButtonToolTipChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleBar::*)()>(_a, &TitleBar::numDockWidgetsChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuickItem**>(_v) = _t->titleBarQmlItem(); break;
        case 1: *reinterpret_cast<QQuickItem**>(_v) = _t->titleBarMouseArea(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasIcon(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->closeButtonEnabled(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->floatButtonVisible(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->maximizeButtonVisible(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->minimizeButtonVisible(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->floatButtonToolTip(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isFocused(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->maximizeUsesRestoreIcon(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTitleBarQmlItem(*reinterpret_cast<QQuickItem**>(_v)); break;
        case 4: _t->setCloseButtonEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFloatButtonVisible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KDDockWidgets::QtQuick::TitleBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::QtQuick::TitleBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick8TitleBarE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::TitleBarViewInterface"))
        return static_cast< Core::TitleBarViewInterface*>(this);
    return QtQuick::View::qt_metacast(_clname);
}

int KDDockWidgets::QtQuick::TitleBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtQuick::View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KDDockWidgets::QtQuick::TitleBar::titleBarQmlItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KDDockWidgets::QtQuick::TitleBar::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KDDockWidgets::QtQuick::TitleBar::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KDDockWidgets::QtQuick::TitleBar::isFocusedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KDDockWidgets::QtQuick::TitleBar::closeButtonEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KDDockWidgets::QtQuick::TitleBar::floatButtonVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KDDockWidgets::QtQuick::TitleBar::maximizeButtonVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KDDockWidgets::QtQuick::TitleBar::minimizeButtonVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KDDockWidgets::QtQuick::TitleBar::floatButtonToolTipChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KDDockWidgets::QtQuick::TitleBar::numDockWidgetsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
