/****************************************************************************
** Meta object code from reading C++ file 'Group.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/qtquick/views/Group.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Group.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::QtQuick::Group::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::QtQuick::Group",
        "QML.Element",
        "GroupView",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Created by the framework only.",
        "isMDIChanged",
        "",
        "currentDockWidgetChanged",
        "actualTitleBarChanged",
        "updateConstraints",
        "setMDISize",
        "QSize",
        "closeAllDockWidgets",
        "setStackLayout",
        "QQuickItem*",
        "startMDIResize",
        "tabBar",
        "KDDockWidgets::QtQuick::TabBar*",
        "titleBar",
        "KDDockWidgets::QtQuick::TitleBar*",
        "userType",
        "actualTitleBar",
        "currentIndex",
        "isMDI",
        "tabsAtBottom"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'isMDIChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentDockWidgetChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'actualTitleBarChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateConstraints'
        QtMocHelpers::SlotData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'setMDISize'
        QtMocHelpers::MethodData<void(QSize)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 8 },
        }}),
        // Method 'closeAllDockWidgets'
        QtMocHelpers::MethodData<bool()>(14, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'setStackLayout'
        QtMocHelpers::MethodData<void(QQuickItem *)>(15, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 8 },
        }}),
        // Method 'startMDIResize'
        QtMocHelpers::MethodData<void()>(17, 8, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'tabBar'
        QtMocHelpers::PropertyData<KDDockWidgets::QtQuick::TabBar*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'titleBar'
        QtMocHelpers::PropertyData<KDDockWidgets::QtQuick::TitleBar*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'userType'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'actualTitleBar'
        QtMocHelpers::PropertyData<KDDockWidgets::QtQuick::TitleBar*>(23, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'currentIndex'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'isMDI'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'tabsAtBottom'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Group, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::QtQuick::Group::staticMetaObject = { {
    QMetaObject::SuperData::link<QtQuick::View::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::QtQuick::Group::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Group *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->isMDIChanged(); break;
        case 1: _t->currentDockWidgetChanged(); break;
        case 2: _t->actualTitleBarChanged(); break;
        case 3: _t->updateConstraints(); break;
        case 4: _t->setMDISize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 5: { bool _r = _t->closeAllDockWidgets();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setStackLayout((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 7: _t->startMDIResize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Group::*)()>(_a, &Group::isMDIChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)()>(_a, &Group::currentDockWidgetChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)()>(_a, &Group::actualTitleBarChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDDockWidgets::QtQuick::TitleBar* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KDDockWidgets::QtQuick::TabBar**>(_v) = _t->tabBarObj(); break;
        case 1: *reinterpret_cast<KDDockWidgets::QtQuick::TitleBar**>(_v) = _t->titleBar(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->userType(); break;
        case 3: *reinterpret_cast<KDDockWidgets::QtQuick::TitleBar**>(_v) = _t->actualTitleBar(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->currentIndex(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isMDI(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->tabsAtBottom(); break;
        default: break;
        }
    }
}

const QMetaObject *KDDockWidgets::QtQuick::Group::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::QtQuick::Group::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick5GroupE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::GroupViewInterface"))
        return static_cast< Core::GroupViewInterface*>(this);
    return QtQuick::View::qt_metacast(_clname);
}

int KDDockWidgets::QtQuick::Group::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtQuick::View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KDDockWidgets::QtQuick::Group::isMDIChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KDDockWidgets::QtQuick::Group::currentDockWidgetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KDDockWidgets::QtQuick::Group::actualTitleBarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
