/****************************************************************************
** Meta object code from reading C++ file 'ClassicIndicatorsWindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/qtquick/views/ClassicIndicatorsWindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ClassicIndicatorsWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::QtQuick::IndicatorWindow::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::QtQuick::IndicatorWindow"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IndicatorWindow, qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::QtQuick::IndicatorWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::QtQuick::IndicatorWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IndicatorWindow *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::QtQuick::IndicatorWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::QtQuick::IndicatorWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick15IndicatorWindowE_t>.strings))
        return static_cast<void*>(this);
    return QQuickView::qt_metacast(_clname);
}

int KDDockWidgets::QtQuick::IndicatorWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickView::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Created by the framework only.",
        "indicatorsVisibleChanged",
        "",
        "hoveredGroupRectChanged",
        "currentDropLocationChanged",
        "iconName",
        "loc",
        "active",
        "innerLeftIndicatorVisible",
        "innerRightIndicatorVisible",
        "innerTopIndicatorVisible",
        "innerBottomIndicatorVisible",
        "outterLeftIndicatorVisible",
        "outterRightIndicatorVisible",
        "outterTopIndicatorVisible",
        "outterBottomIndicatorVisible",
        "tabIndicatorVisible",
        "hoveredGroupRect",
        "QRect",
        "currentDropLocation",
        "KDDockWidgets::DropLocation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'indicatorsVisibleChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hoveredGroupRectChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentDropLocationChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'iconName'
        QtMocHelpers::MethodData<QString(int, bool) const>(11, 8, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 12 }, { QMetaType::Bool, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'innerLeftIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'innerRightIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'innerTopIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'innerBottomIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'outterLeftIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'outterRightIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'outterTopIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'outterBottomIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'tabIndicatorVisible'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'hoveredGroupRect'
        QtMocHelpers::PropertyData<QRect>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'currentDropLocation'
        QtMocHelpers::PropertyData<KDDockWidgets::DropLocation>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<ClassicDropIndicatorOverlay, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE[] = {
    QMetaObject::SuperData::link<KDDockWidgets::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ClassicDropIndicatorOverlay *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->indicatorsVisibleChanged(); break;
        case 1: _t->hoveredGroupRectChanged(); break;
        case 2: _t->currentDropLocationChanged(); break;
        case 3: { QString _r = _t->iconName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ClassicDropIndicatorOverlay::*)()>(_a, &ClassicDropIndicatorOverlay::indicatorsVisibleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClassicDropIndicatorOverlay::*)()>(_a, &ClassicDropIndicatorOverlay::hoveredGroupRectChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClassicDropIndicatorOverlay::*)()>(_a, &ClassicDropIndicatorOverlay::currentDropLocationChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->innerLeftIndicatorVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->innerRightIndicatorVisible(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->innerTopIndicatorVisible(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->innerBottomIndicatorVisible(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->outterLeftIndicatorVisible(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->outterRightIndicatorVisible(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->outterTopIndicatorVisible(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->outterBottomIndicatorVisible(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->tabIndicatorVisible(); break;
        case 9: *reinterpret_cast<QRect*>(_v) = _t->hoveredGroupRect(); break;
        case 10: *reinterpret_cast<KDDockWidgets::DropLocation*>(_v) = _t->currentDropLocation(); break;
        default: break;
        }
    }
}

const QMetaObject *KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets7QtQuick27ClassicDropIndicatorOverlayE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::ClassicIndicatorWindowViewInterface"))
        return static_cast< Core::ClassicIndicatorWindowViewInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::indicatorsVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::hoveredGroupRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KDDockWidgets::QtQuick::ClassicDropIndicatorOverlay::currentDropLocationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
