/****************************************************************************
** Meta object code from reading C++ file 'DragController_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../KDDockWidgets-2.4.1/src/core/DragController_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DragController_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::State::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::State"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<State, qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::State::staticMetaObject = { {
    QMetaObject::SuperData::link<Core::Object::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::State::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<State *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::State::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::State::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core5StateE_t>.strings))
        return static_cast<void*>(this);
    return Core::Object::qt_metacast(_clname);
}

int KDDockWidgets::Core::State::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Core::Object::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::MinimalStateMachine::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::MinimalStateMachine"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MinimalStateMachine, qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::MinimalStateMachine::staticMetaObject = { {
    QMetaObject::SuperData::link<Core::Object::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::MinimalStateMachine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MinimalStateMachine *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::MinimalStateMachine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::MinimalStateMachine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core19MinimalStateMachineE_t>.strings))
        return static_cast<void*>(this);
    return Core::Object::qt_metacast(_clname);
}

int KDDockWidgets::Core::MinimalStateMachine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Core::Object::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::DragController::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::DragController",
        "State",
        "State_None",
        "State_PreDrag",
        "State_Dragging"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(1, 1, QMC::EnumFlags{}).add({
            {    2, State::State_None },
            {    3, State::State_PreDrag },
            {    4, State::State_Dragging },
        }),
    };
    return QtMocHelpers::metaObjectData<DragController, qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::DragController::staticMetaObject = { {
    QMetaObject::SuperData::link<MinimalStateMachine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::DragController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DragController *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::DragController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::DragController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core14DragControllerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "EventFilterInterface"))
        return static_cast< EventFilterInterface*>(this);
    return MinimalStateMachine::qt_metacast(_clname);
}

int KDDockWidgets::Core::DragController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MinimalStateMachine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::StateBase::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::StateBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateBase, qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::StateBase::staticMetaObject = { {
    QMetaObject::SuperData::link<State::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::StateBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::StateBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::StateBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateBaseE_t>.strings))
        return static_cast<void*>(this);
    return State::qt_metacast(_clname);
}

int KDDockWidgets::Core::StateBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = State::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::StateNone::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::StateNone"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateNone, qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::StateNone::staticMetaObject = { {
    QMetaObject::SuperData::link<StateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::StateNone::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateNone *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::StateNone::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::StateNone::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core9StateNoneE_t>.strings))
        return static_cast<void*>(this);
    return StateBase::qt_metacast(_clname);
}

int KDDockWidgets::Core::StateNone::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::StatePreDrag::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::StatePreDrag"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StatePreDrag, qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::StatePreDrag::staticMetaObject = { {
    QMetaObject::SuperData::link<StateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::StatePreDrag::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StatePreDrag *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::StatePreDrag::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::StatePreDrag::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core12StatePreDragE_t>.strings))
        return static_cast<void*>(this);
    return StateBase::qt_metacast(_clname);
}

int KDDockWidgets::Core::StatePreDrag::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::StateDragging::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::StateDragging"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateDragging, qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::StateDragging::staticMetaObject = { {
    QMetaObject::SuperData::link<StateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::StateDragging::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateDragging *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::StateDragging::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::StateDragging::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core13StateDraggingE_t>.strings))
        return static_cast<void*>(this);
    return StateBase::qt_metacast(_clname);
}

int KDDockWidgets::Core::StateDragging::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StateBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t {};
} // unnamed namespace

template <> constexpr inline auto KDDockWidgets::Core::StateInternalMDIDragging::qt_create_metaobjectdata<qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDDockWidgets::Core::StateInternalMDIDragging"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateInternalMDIDragging, qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDDockWidgets::Core::StateInternalMDIDragging::staticMetaObject = { {
    QMetaObject::SuperData::link<StateBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>.metaTypes,
    nullptr
} };

void KDDockWidgets::Core::StateInternalMDIDragging::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateInternalMDIDragging *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDDockWidgets::Core::StateInternalMDIDragging::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDDockWidgets::Core::StateInternalMDIDragging::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KDDockWidgets4Core24StateInternalMDIDraggingE_t>.strings))
        return static_cast<void*>(this);
    return StateBase::qt_metacast(_clname);
}

int KDDockWidgets::Core::StateInternalMDIDragging::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StateBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
